package infomat.algorithms.labeling;

import infomat.vectorspace.*;

import java.util.*;


/**
 * Reduces IObjectGroups and IObjectGroupings compared to an other.
 * The class can reduce both in order of the IObject:s in each 
 * IObjectGroup or in random order.
 */
public class SizeReducer{

    /**
     * Note: does not handle missing zero-sized groups!!!
     */
    public static IObjectGrouping reduceGroupingToSizeOfOtherGroupingInOrder(IObjectGrouping ioging, IObjectGrouping redIOGING, IMatrix matrix){ 
	Vector<IObjectGroup> groups = ioging.getGroups();
	Vector<IObjectGroup> redGroups = redIOGING.getGroups();
	Vector<IObjectGroup> newGroups = new Vector<IObjectGroup>();
	
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup newGroup = 
		reduceGroupToSizeOfOtherGroupInOrder(groups.elementAt(i), redGroups.elementAt(i));
	    newGroups.add(newGroup);
	}

	
	IObjectSet ioset = matrix.getRowIObjectSet();
	if(!ioset.containsIObjectGrouping(ioging))
	    ioset = matrix.getColumnIObjectSet();
	
	String name = "Order Size Reduced "+ioging.getName();
	IObjectGrouping newIOGING = 
	    new IObjectGrouping(name);
	int i=1;
	while(!ioset.setGrouping(newIOGING)){
	    newIOGING.setName(name+" ("+i+")");
	    i++;
	}

	newIOGING.addAllGroups(newGroups);
	return newIOGING;
    }
    
    /**
     * Makes a new IObjectGroup that consists of 
     * the top IObject:s of "iog". As many as there are in the "redIOG".
     * Currently produces error if the group to reduce is smaller than
     * the group with the proposed size.
     */
    public static IObjectGroup reduceGroupToSizeOfOtherGroupInOrder(IObjectGroup iog, IObjectGroup redIOG){
	int size = redIOG.getOrdered().size();
	Vector<IObject> ordered = 
	    new Vector<IObject>(iog.getOrdered().subList(0, size));
	IObjectGroup newIOG = new IObjectGroup("Order Size Reduced "+iog.getName());
	newIOG.addAll(ordered);
	return newIOG;
    }


    /*************************************/

    /**
     * Note: does not handle missing zero-sized groups!!!
     */
    public static IObjectGrouping reduceGroupingToSizeOfOtherGroupingRandomly(IObjectGrouping ioging, IObjectGrouping redIOGING, IMatrix matrix){ 
	Vector<IObject> ios = ioging.getOrdered();
	Vector<IObjectGroup> groups = ioging.getGroups();
	Vector<IObjectGroup> redGroups = redIOGING.getGroups();
	Vector<IObjectGroup> newGroups = new Vector<IObjectGroup>();

	for(int i=0; i<groups.size(); i++){
	    IObjectGroup newGroup = 
		reduceGroupToSizeOfOtherGroupRandomly(groups.elementAt(i), redGroups.elementAt(i));
	    newGroups.add(newGroup);
	}
	
	IObjectSet ioset = matrix.getRowIObjectSet();
	if(!ioset.containsIObjectGrouping(ioging))
	    ioset = matrix.getColumnIObjectSet();
	
	String name = "Randomly Size Reduced "+ioging.getName();
	IObjectGrouping newIOGING = 
	    new IObjectGrouping(name);
	int i=1;
	while(!ioset.setGrouping(newIOGING)){
	    newIOGING.setName(name+" ("+i+")");
	    i++;
	}

	newIOGING.addAllGroups(newGroups);
	return newIOGING;
    }
    
    /**
     * Makes a new IObjectGroup that consists of 
     * the top IObject:s of "iog". As many as there are in the "redIOG".
     * Currently produces error if the group to reduce is smaller than
     * the group with the proposed size.
     */
    public static IObjectGroup reduceGroupToSizeOfOtherGroupRandomly(IObjectGroup iog, IObjectGroup redIOG){
	int size = redIOG.getOrdered().size();
	Vector<IObject> newVec = new Vector<IObject>();
	Vector<IObject> copyVec = new Vector<IObject>(iog.getOrdered());
	for(int i=0; i<size; i++){
	    int nr = (int) (Math.random() * copyVec.size());
	    newVec.add(copyVec.remove(nr));
	}
	
	IObjectGroup newIOG = new IObjectGroup("Randomly Size Reduced "+iog.getName());
	newIOG.addAll(newVec);
	return newIOG;
    }







}
