package infomat.io;

import infomat.vectorspace.*;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

/**
 * For reading a set of texts from a directory (and its subdirectories).
 * 
 * @version          080211
 * @author           Magnus Rosell
 */
public class TextDirectoryReader{

    private IMatrix matrix;
    private TextWordIMatrixConstructor twmc;
    private static int idCounter=1;

    public static TextFileFilter tff = new TextFileFilter();
    public static AllFileFilter aff = new AllFileFilter();

    public TextDirectoryReader(){
	this.twmc = new TextWordIMatrixConstructor();
    }

    public TextDirectoryReader(TextWordIMatrixConstructor twmc){
	this.twmc = twmc;
    }

    /**
     * Loads a IMatrix from a directory.
     */
    public IMatrix load(File file) {
	//	matrix = new IMatrix("Words", "Texts");
	    
	recurseFile(file);

	//	return matrix;
	return twmc.getIMatrix();
    }

    private void recurseFile(File file) {
	if(file.isDirectory()){
	    File[] list = file.listFiles(aff);
	    for(int i=0; i<list.length; i++){
		recurseFile(list[i]);
	    }
	}else{
	    doOneFile(file);
	}
    }

    private HashMap<String, IObject> stringWordObjectMap = 
	new HashMap<String, IObject>();

    private void doOneFile(File file){

	try {

	    twmc.addText(file.getName(), file.getPath());

	    /*
	    IObject textObject = 
		new IObject(file.getName(), file.getPath(), idCounter);
	    idCounter++;
	    matrix.addRowIObject(textObject);
	    
	    IObjectSet wordSet = matrix.getColumnIObjectSet();
	    */

	    BufferedReader in = 
		new BufferedReader(new FileReader(file));
	    String s=in.readLine();
	    while(s!=null){
		String[] words = s.split(" ");
		for(int i=0; i<words.length; i++){

		    twmc.addWord(words[i], null, 1, 1, 1);


		    /*
		    IObject wordObject = stringWordObjectMap.get(words[i]);
		    if(wordObject==null){
			wordObject = new IObject(words[i], idCounter);
			idCounter++;
			stringWordObjectMap.put(words[i], wordObject);
			matrix.addColumnIObject(wordObject);
		    }
		    IMatrixCell mc = matrix.getCell(textObject, wordObject);
		    if(mc==null){
			mc = new IMatrixCell(textObject, wordObject, 1, 1);
			matrix.addCell(mc);
		    }else{
			mc.addOneToLocalCount();
		    }
		    */
		}

		s=in.readLine();
	    }
	}catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }
}


class TextFileFilter implements FileFilter{

    public boolean accept(File pathname){
	if(pathname.isDirectory())
	    return true;
	if(pathname.getName().endsWith(".txt") ||
	   pathname.getName().endsWith(".TXT"))
	    return true;
	return false;
    }

}

class AllFileFilter implements FileFilter{

    public boolean accept(File pathname){
	return true;
    }

}
