package infomat.io;

import infomat.vectorspace.*;

import mro.util.PathHelper;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * For loading a token file into a matrix with the texts as rows
 * and words as columns. Do not use in combination with IObjectGroupingIO.
 * The later shares id:s with CorpusIO that are not present in the TokenFiles.
 * 
 * @version          070726
 * @author           Magnus Rosell
 */
public class TokenFileTextWordIO{

    // The InfomatModel that is loaded/saved
    //    private InfomatModel im;
    private IMatrix matrix;
    private TextWordIMatrixConstructor twmc;


    public TokenFileTextWordIO(){ //InfomatModel im){
	this.twmc = new TextWordIMatrixConstructor(null, null);
    }


    public TokenFileTextWordIO(TextWordIMatrixConstructor twmc){
	this.twmc = twmc;
    }



    /**
     * Loads a IMatrix from file in XML format
     */
    public IMatrix load(File file) {
	try {
	    BufferedReader in = 
		new BufferedReader(new FileReader(file));
	    InputSource source = new InputSource(in);
	    return load(source);
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	return null;
    }

    public IMatrix loadCompressed(File file) {
	try {
	    // Create a ZipInputStream to read the zip file            
	    ZipInputStream zis = new ZipInputStream(new BufferedInputStream(new FileInputStream(file)));
	    InputSource source = new InputSource(zis);
	    ZipEntry entry;
	    if((entry = zis.getNextEntry()) != null) {
		if(!entry.isDirectory()) {
		    return load(source);
		}
	    }
	    zis.close();
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	return null;

    }

    private IMatrix load(InputSource source) {
	try {

	    //	    IMatrix m = new IMatrix("Words", "Documents");

	    // Create parser+contentHandler and read the XML document
	    XMLReader parser = XMLReaderFactory.createXMLReader();
	    TokenFileTextWordHandler handler = 
		new TokenFileTextWordHandler(twmc);
	    //		new TokenFileTextWordHandler(m);
	    parser.setContentHandler(handler);
	    parser.parse(source);

	    IMatrix m = twmc.getIMatrix();
	    m.getRowIObjectSet().sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();
	    m.getColumnIObjectSet().sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();

	    return m;

	    //	    corpus.setUniqueIdentifier(handler.getUniqueIdentifier());
	} catch (Exception e) {
	    System.err.println(e);
	    e.printStackTrace();
	}
	return null;
    }


    /**
     * Exports the IMatrix in XML format
     */

    /*
      public void save(IMatrix m, File file){
      matrix=m;
      try {
      PrintWriter out =
      new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));
      save(out);
      out.close();
      }
      catch(IOException ioE) {
      System.out.println(ioE);
      ioE.printStackTrace();
      }
      }
    */

    /**
     * Exports the IMatrix in XML format
     * The file is saved in compressed .zip-format.
     */

    /*
      public void saveCompressed(IMatrix m, String fileName) { 
      matrix=m;
      try {
      // Create zip compressed output stream, zip file entry and PrintWriter
      ZipOutputStream targetStream = new ZipOutputStream(new FileOutputStream(fileName+".xml.zip"));
      File file = new File(fileName);                        
      targetStream.putNextEntry(new ZipEntry(file.getName() + ".xml"));
      PrintWriter out = new PrintWriter(targetStream);
      save(out);            
      // Close compressed output stream and PrintWriter
      out.flush();
      targetStream.closeEntry();
      targetStream.close();
      targetStream = null;
      out.close();
      }
      catch (java.io.IOException ioE) {
      System.out.println(ioE);
      ioE.printStackTrace();
      }
      }
    */


    /*
      public void printDTD(PrintWriter out) {
      //We use the xml-format specified in corpus.dtd
      //out.println("<?xml version='1.0' encoding='ISO-8859-1'?>");

      out.println("<?xml version='1.0' encoding='ISO-8859-1' standalone=\"yes\"?>");

      //	out.println("<?xml-stylesheet type=\"text/xsl\" href=\""+corpus.sourcePath+"default/Corpus.xsl\"?>");

      out.println("<!DOCTYPE InfomatIMatrix [");

      out.println("<!ELEMENT InfomatIMatrix (Rows, Columns, Cells)>");
      out.println("<!ELEMENT Rows (IObjectSet)>");
      out.println("<!ELEMENT Columns (IObjectSet)>");

      out.println("<!--A Set of IObjects-->");
      out.println("<!ELEMENT IObjectSet (SetName, io*)>");
      out.println("<!ELEMENT SetName (#PCDATA)>");

      out.println("<!ELEMENT io (s, d?, id)> <!--A IObject-->");
      out.println("<!ELEMENT s (#PCDATA)> <!--The objects description-->");
      out.println("<!ELEMENT k (#PCDATA)> <!--Kind-->");
      out.println("<!ELEMENT id (#PCDATA)> <!--Object ID-->");


      out.println("<!--The Non-Zero Cells of the IMatrix-->");
      out.println("<!ELEMENT IMatrixCells (mc*)>");
      out.println("<!ELEMENT mc (rid, cid, c?, lw?, w)> <!--IMatrix Cell-->");
      out.println("<!ELEMENT rid (#PCDATA)> <!--Row Object ID-->");
      out.println("<!ELEMENT cid (#PCDATA)> <!--Column Object ID-->");
      out.println("<!ELEMENT c (#PCDATA)> <!--Frequency-->");
      out.println("<!ELEMENT lw (#PCDATA)> <!--Local Weight-->");
      out.println("<!ELEMENT w (#PCDATA)> <!--Weight-->");

      out.println("]>");
      }
    */


    /*
      private void save(PrintWriter out) {

      printDTD(out);

      out.println("<InfomatMatrx>");


      out.println("<Rows>");
      out.println("<IObjectSet>");
      saveIObjectSet(matrix.getRowIObjectSet(), out);
      out.println("</IObjectSet>");
      out.println("</Rows>");

      out.println("<Columns>");
      out.println("<IObjectSet>");
      saveIObjectSet(matrix.getColumnIObjectSet(), out);
      out.println("</IObjectSet>");
      out.println("</Columns>");

      out.println("<IMatrixCells>");
      saveCells(matrix, out);
      out.println("</IMatrixCells>");

      out.println("</InfomatMatrx>");

      out.flush();

      }
    */


    /*
      private void saveIObjectSet(IObjectSet set, PrintWriter out){
      out.println("\t<SetName>"+set.getName()+"</SetName>");
      Vector<IObject> objs = set.getOrdered();
      for(int i=0; i<objs.size(); i++){
      out.println("\t<io>");
      IObject io = objs.elementAt(i);
      out.println("\t\t<s>"+io.getString()+"</s>");
      //	    out.println("<k>"+io.getKind()+"</k>");
      out.println("\t\t<id>"+io.getID()+"</id>");
      out.println("\t</io>");
      }
      }
    */

    /*
      private void saveCells(IMatrix matrix, PrintWriter out){
      Vector cells = matrix.getAllCells();
      for(int i=0; i<cells.size(); i++){
      IMatrixCell mc = (IMatrixCell) cells.elementAt(i);
      out.println("\t<mc>");
      out.println("\t\t<rid>"+mc.getRowIObject().getID()+"</rid>");
      out.println("\t\t<cid>"+mc.getColumnIObject().getID()+"</cid>");
      //	    out.println("<c>"+mc.getLocalCount()+"</c>");
      //	    out.println("<lw>"+mc.getLocalWeight()+"</lw>");
      out.println("\t\t<w>"+mc.getWeight()+"</w>");
      out.println("\t</mc>");
      }
      }
    */


    private class TokenFileTextWordHandler extends DefaultHandler {
	private StringBuffer buffer;

	private TextWordIMatrixConstructor twmc;
	//private IMatrix m;

	/************/
	private int id=0;
	private int totalLocalCount=0;
	private IObject textObject;
	private HashMap<String, Integer> stringNumberMap;
	private HashMap<String, IObject> stringIObjectMap;
	private boolean solidCompound=false;
	private boolean parts=false;
	private String file=null;
	private String textTitle=null;
	private String documentPath=null;

	public TokenFileTextWordHandler(TextWordIMatrixConstructor twmc){//IMatrix m){ 
	    //	    this.m=m;
	    this.twmc=twmc;
	}

	public void startElement (String namespaceURI, String localName,
				  String qualifiedName, Attributes atts) {
	    buffer = new StringBuffer();

	    
	    if(qualifiedName.equals("tokenfile")){
		stringIObjectMap = new HashMap<String, IObject>();
		documentPath=null;
	    }else if(qualifiedName.equals("document")){
		totalLocalCount=0;
		stringNumberMap = new HashMap<String, Integer>();
		file=null;
		textTitle=null;
	    }else if(qualifiedName.equals("sc")){
		solidCompound=true;
	    }else if(qualifiedName.equals("ps")){
		parts=true;
	    }

	}

	public void endElement (String namespaceURI, String localName, String qualifiedName) {

	    if(qualifiedName.equals("documentpath")){
		documentPath=PathHelper.convertToCurrentFileSeparator(buffer.toString().trim());
	    }else if(qualifiedName.equals("file")){
		file = PathHelper.convertToCurrentFileSeparator(buffer.toString().trim());
		if(documentPath!=null && !file.startsWith(documentPath)){
		    file = documentPath+file;
		}
	    }else if(qualifiedName.equals("t")){
		textTitle = buffer.toString().trim();
	    }else if(qualifiedName.equals("sc")){
		solidCompound=false;
	    }else if(qualifiedName.equals("ps")){
		parts=false;
	    }else if(qualifiedName.equals("s")){
		if(!solidCompound || (solidCompound && parts)){
		    String string = buffer.toString().trim();
		    //		    System.out.println("    "+string);
		    Integer n=stringNumberMap.get(string);
		    if(n!=null)
			n++;
		    else
			n=1;
		    totalLocalCount++;
		    stringNumberMap.put(string,n);
		}
	    }else if(qualifiedName.equals("document")){
		if(textTitle!=null){
		    //		    System.out.println("title: "+textTitle);
		    twmc.addText(textTitle, file);
		}else{
		    //		    String[] parts = file.split("["+File.separator+"]");
		    String[] parts = 
			PathHelper.splitFileNameAtSeparators(file);
		    String name = parts[parts.length-1];
		    twmc.addText(name, file);
		    //		    System.out.println("name: "+name);
		}
		//		IObjectSet cols = m.getColumnIObjectSet();
		Iterator<String> stringIt = stringNumberMap.keySet().iterator();
		while(stringIt.hasNext()){
		    String s = stringIt.next();

		    Integer n = stringNumberMap.get(s);
		    twmc.addWord(s, null, n, ((float) n)/totalLocalCount, ((float) n)/totalLocalCount);

		    /*
		    IObject wordObject = stringIObjectMap.get(s);
		    if(wordObject==null){
			wordObject = new IObject(s, id++);
			stringIObjectMap.put(s, wordObject);
			cols.add(wordObject);
		    }
		    Integer n = stringNumberMap.get(s);
		    IMatrixCell mc = new IMatrixCell(textObject, wordObject, n, (float) n/totalLocalCount);
		    m.addCell(mc);
		    */
		}

	    }

	    /*
	    if(qualifiedName.equals("tokenfile")){
		Iterator<String> it = stringIObjectMap.keySet().iterator();
		int i=0;
		while(it.hasNext()){
		    System.out.println(i+", "+it.next());
		    i++;
		}
	    }
	    */
	}

	public void characters (char ch[], int start, int length) {
	    if(buffer != null)
		buffer.append(ch,start,length);
	}      
    }




}
