package infomat.mro;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;

import mro.util.*;
import mro.util.properties.*;

import java.io.*;
import java.util.Vector;

import org.xml.sax.SAXException;




/****************/

// java -cp .:classes/ -Xms1024m -Xmx1024m infomat.FileConverter -tokenFile "/afs/nada.kth.se/home/f/u1ya3frf/Korpus/tokenFiles/jtteLitenKorpus.corpus.tokenFile.xml" -resultPath "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/res/"

// java -cp .:classes/ -Xms1024m -Xmx1024m infomat.FileConverter -documentGrouping "/afs/nada.kth.se/home/f/u1ya3frf/Code/Infomat/example/doc_grouping-categorization.xml" "/afs/nada.kth.se/home/f/u1ya3frf/Code/Infomat/example/corpus.xml" -resultPath "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/res/"





/**
 * Class for converting Infomat file types.
 * For natural reasons the conversion is not complete,
 * i.e. all file types can not be converted to all others.
 * In fact it is only possible to convert to
 * the Infomat types, that is IMatrix and IObjectGrouping.
 * The class supposes that the groupings are along the rows.
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/ -Xms1024m -Xmx1024m infomat.FileConverter</code>
 *
 * @version          080318
 * @author           Magnus Rosell
 *
 */
public class FileConverter implements HasProperties{

    private Properties properties = null;
    private String separator=null;

    public FileConverter(){
	this.properties = getDefaultProperties();
	separator = System.getProperties().getProperty("file.separator");

    }

    private Vector<PropertyValueDescription> getAllFileTypes(){
	Vector<PropertyValueDescription> descriptions = 
	    new Vector<PropertyValueDescription>();

	PropertyValueDescription pvd = 
	    new PropertyValueDescription("1", "\"Matrix File\"");
	descriptions.add(pvd);
	descriptions.add(pvd);
	pvd = new PropertyValueDescription("2", "\"Token File\"");
	descriptions.add(pvd);
	pvd = new PropertyValueDescription("3", "\"Corpus File\" (older file format)");
	descriptions.add(pvd);
	pvd = new PropertyValueDescription("4", "\"IObjectGrouping File\"");
	descriptions.add(pvd);
	pvd = new PropertyValueDescription("5", "\"Document Grouping File\" (older file format, only possible to use in conjunction with a \"Corpus File\"");
	descriptions.add(pvd);

	return descriptions;
    }


    public Properties getDefaultProperties(){

	Properties properties = 
	    new Properties("File Converter Properties", CurrentClassGetter.getClassName());
		
	try{
			
	    /****************/
	    Property p = new Property("In File Type", 0, "What kind of file for input");
	    PropertyValueDescription pvd = new PropertyValueDescription("0", " A directory of files are added recursively");
	    p.addValueDescription(pvd);

	    Vector<PropertyValueDescription> descriptions = getAllFileTypes();
	    for(int i=0; i<descriptions.size(); i++)
		p.addValueDescription(descriptions.elementAt(i));
	    properties.addProperty(p);

	    p = new Property("In File", null, "The in file");
	    properties.addProperty(p);

	    p = new Property("Secondary In File", null, "The secondary in file");
	    properties.addProperty(p);


	    /****************/
	    p = new Property("Out File Type", 0, "What kind of file for output");

	    descriptions = getAllFileTypes();
	    for(int i=0; i<descriptions.size(); i++)
		p.addValueDescription(descriptions.elementAt(i));
	    properties.addProperty(p);

	    p = new Property("Result Path", null, "The Path for output");
	    properties.addProperty(p);

			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }


    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }




    public static void main(String[] args) throws IOException, PropertyNotFoundException{
	FileConverter fc = new FileConverter();
	fc.doIt(args);
    }

    public void doIt(String[] args) throws IOException, PropertyNotFoundException{

	checkInput(args);

	readFiles();

	writeFiles();

    }


    private void badInput(String comment){
	System.out.println("Bad input!");
	System.out.println(comment);
	System.exit(0);
    }

    
    private void checkInput(String[] args) throws IOException, PropertyNotFoundException{

	if(args.length==0)
	    badInput("To few arguments.");



	for(int i=0; i<args.length; i++){
	    if(args[i].equals("-inpath")){
		setProperty("In File", args[i+1]);
		setProperty("In File Type", 0);
	    }else if(args[i].equals("-matrix")){
		setProperty("In File", args[i+1]);
		setProperty("In File Type", 1);
	    }else if(args[i].equals("-tokenFile")){
		setProperty("In File", args[i+1]);
		setProperty("In File Type", 2);
	    }else if(args[i].equals("-corpus")){
		setProperty("In File", args[i+1]);
		setProperty("In File Type", 3);
	    }else if(args[i].equals("-iobjectGrouping")){
		setProperty("In File", args[i+1]);
		setProperty("Secondary In File", args[i+2]);
		setProperty("In File Type", 4);
	    }else if(args[i].equals("-documentGrouping")){
		setProperty("In File", args[i+1]);
		setProperty("Secondary In File", args[i+2]);
		setProperty("In File Type", 5);
	    }else if(args[i].equals("-resultPath")){
		setProperty("Result Path", args[i+1]);
	    }
	}
	
	    


	String inFile = (String) getPropertyValue("In File");
	if(inFile==null)
	    badInput("No in file!");
	System.out.println("In File: "+inFile);

	// Only the following outputs
	int inFileType = (Integer) 
	    getPropertyValue("In File Type");
	if(inFileType==4 || inFileType==5)
	    setProperty("Out File Type", 4);
	else
	    setProperty("Out File Type", 1);

	String resultPath = (String) getPropertyValue("Result Path");
	if(resultPath==null)
	    badInput("No output path");

	System.out.println("Result Path "+resultPath);
	File path = new File(resultPath);
	if(!path.exists())
	    path.mkdirs();
    }


    private IMatrix matrix = null;
    private IObjectGrouping grouping = null;

    private void readFiles(){

	int inFileType = (Integer) 
	    getPropertyValue("In File Type");
	String inFile = (String) 
	    getPropertyValue("In File");
	String secondaryInFile = (String) 
	    getPropertyValue("Secondary In File");
	
	File file = new File(inFile);

	// Lsa in matris
	if(inFileType==0){
	    TextDirectoryReader tdr = new TextDirectoryReader();
	    matrix = tdr.load(file);
	}else if(inFileType==1){
	    IMatrixIO iomatrix = new IMatrixIO();
	    matrix = iomatrix.load(file);
	}else if(inFileType==2){
	    TokenFileTextWordIO tokenIO = new TokenFileTextWordIO();
	    matrix = tokenIO.load(file);
	}else if(inFileType==3){
	    CorpusIO ioCorpus = new CorpusIO();
	    matrix = ioCorpus.load(file);
	}

	// Lsa in Grouping
	else if(inFileType==4){
	    if(secondaryInFile==null)
		badInput("No secondary in file!");
	    File file2 = new File(secondaryInFile);
	    IMatrixIO iomatrix = new IMatrixIO();
	    matrix = iomatrix.load(file);

	    boolean row=true; // row categorization
	    IObjectGroupingIO iobjectGroupingIO = new IObjectGroupingIO();
	    grouping = iobjectGroupingIO.load(row, matrix, file);
	}else if(inFileType==5){
	    if(secondaryInFile==null)
		badInput("No secondary in file!");
	    File file2 = new File(secondaryInFile);
	    CorpusIO ioCorpus = new CorpusIO();
	    matrix = ioCorpus.load(file2);
	    DocumentGroupingIO ioDocumentGrouping = 
		new DocumentGroupingIO(matrix);
	    grouping = ioDocumentGrouping.load(file);
	}else
	    badInput("No input file");



    }

    private void writeFiles(){
	int outFileType = (Integer) 
	    getPropertyValue("Out File Type");
	String resPathName = (String) 
	    getPropertyValue("Result Path");

	// Skriva ut matris

	// Can not write to a directory structure.

	if(outFileType==1){
	    String outFileName = resPathName+separator+"matrix.xml";
	    System.out.println("Out File Name: "+outFileName);
	    File file = new File(outFileName);
	    IMatrixIO iomatrix = new IMatrixIO();
	    iomatrix.save(matrix, file);
	}

	// Can not write tokenFile or corpus-file
	/*
	  else if(outFileType==2){
	  String outFileName = resPathName+separator+"tokenFile.xml";
	  System.out.println("Out File Name: "+outFileName);
	  File file = new File(outFileName);
	  TokenFileTextWordIO tokenIO = new TokenFileTextWordIO();
	  tokenIO.save(matrix, file);
	  }else if(outFileType==3){
	  String outFileName = resPathName+separator+"corpus.xml";
	  System.out.println("Out File Name: "+outFileName);
	  File file = new File(outFileName);
	  CorpusIO ioCorpus = new CorpusIO();
	  ioCorpus.save(file);
	  }
	*/

	// Skriva ut Grouping

	else if(outFileType==4){
	    String outFileName = resPathName+separator+"matrix.xml";
	    System.out.println("Out File Name: "+outFileName);
	    File file = new File(outFileName);

	    String outFileName2 = resPathName+separator+"grouping.xml";
	    System.out.println("Out File Name 2: "+outFileName2);
	    File file2 = new File(outFileName2);

	    IMatrixIO iomatrix = new IMatrixIO();
	    iomatrix.save(matrix, file);

	    boolean row=true; 
	    IObjectGroupingIO iobjectGroupingIO = new IObjectGroupingIO();
	    iobjectGroupingIO.save(grouping, matrix, file2);
	}
	
	// Can not write documentgrouping-file
	/*
	  else if(outFileType==5){
	  if(secondaryInFile==null)
	  badInput("No secondary in file!");
	  File file2 = new File(secondaryInFile);
	  CorpusIO ioCorpus = new CorpusIO();
	  matrix = ioCorpus.load(file2);
	  DoucmentGroupingIO ioDocumentGrouping = 
	  new DocumentGroupingIO(matrix);
	  grouping = ioDocumentGrouping.load(file);
	  }
	*/
	else
	    badInput("No output file");




    }

}
