package infomat.view.algorithms;

import infomat.algorithms.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.weighting.*;
import infomat.view.*;
import infomat.view.grouping.settings.*;
import infomat.view.picture.*;
import infomat.view.util.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;
import mro.util.TracePrinter;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;

import javax.swing.border.*;
import javax.swing.filechooser.FileFilter;

import java.util.*;
import java.io.*;

import javax.swing.*;



public class MatrixWeightingFrame extends JFrame implements ActionListener{

    public Dimension MATRIXWEIGHTINGFRAME_PREFERRED_SIZE = 
	new Dimension(300, 500);

    private IMatrix matrix;
    private InfomatModel model;
    private InfomatView view;

    private SmallButton apply;
    private SmallButton cancel;

    private JFileChooser fileChooser;

    //    private TFIDFWeighter weighter = new TFIDFWeighter();
    private Weighter weighter;

    private JPanel mainPanel;
    private JPanel infoAndButtons;
    private MatrixWeightingSelectionPanel matrixWeightingSelectionPanel;

    private boolean printOn=true;
    private TracePrinter tp = new TracePrinter(printOn);

	
    public MatrixWeightingFrame(IMatrix matrix, InfomatModel model,
			     InfomatView view){
	super("Matrix Weighting");
	this.matrix=matrix;
	this.model = model;
	this.view = view;

	if(matrix==null){
	    dispose();
	    return;
	}

	String fileChooserPath = (String) model.getPropertyValue("Matrix File Path");
	fileChooser = new JFileChooser(fileChooserPath);
	fileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "XML files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".xml");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });

	Container container = getContentPane();
	container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));


	//		apply = new SmallButton("Apply");
	//		JButton dummyButton = new JButton("Cancel");
	//		int height = (int) dummy.getPreferredSize().getHeight();
	//		int width = (int) dummyButton.getPreferredSize().getWidth();
	//		Dimension buttonSize = new Dimension(width, height);

	//		apply.addActionListener(this);
	//		add(apply);


	infoAndButtons = new JPanel();
	infoAndButtons.setLayout(new GridLayout(2,1));
	JLabel info = new JLabel("Alter properties below and press \"Apply\"");
	infoAndButtons.add(info);

	
	matrixWeightingSelectionPanel = 
	    new MatrixWeightingSelectionPanel(this);

	weighter = matrixWeightingSelectionPanel.getSelectedWeighter();

	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	apply = new SmallButton("Apply");
	apply.addActionListener(this);
	buttonPanel.add(apply);
	cancel = new SmallButton("Cancel");
	cancel.addActionListener(this);
	buttonPanel.add(cancel);
	infoAndButtons.add(buttonPanel);
	infoAndButtons.setBorder(new TitledBorder("Weight Matrix"));

	setVisible(true);
	setPreferredSize(MATRIXWEIGHTINGFRAME_PREFERRED_SIZE);

	mainPanel = new JPanel();
	mainPanel.setPreferredSize(MATRIXWEIGHTINGFRAME_PREFERRED_SIZE);
	mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

	putAll();
	add(mainPanel);
	pack();
    }
    
    public void putAll(){
	mainPanel.removeAll();

	mainPanel.add(infoAndButtons);

	mainPanel.add(matrixWeightingSelectionPanel);

	PropertiesPanel pp = new PropertiesPanel(weighter);
	mainPanel.add(pp);

	mainPanel.revalidate();
	mainPanel.repaint();
	repaint();
    }




    public void setWeighter(Weighter weighter){
	this.weighter=weighter;
	putAll();
    }


    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    if(source == apply){
		//				ProgressFrame pf = new ProgressFrame("Clustering - "+algorithm.getAlgorithmName());
		weighter.weight(matrix);
		
		view.forceChange();
		view.getOverview().forceChange();
		model.update();
		
	    }else if(source == cancel){
		dispose();
	    }
	}
    }
}
