package infomat.view.picture;

import java.io.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;

import infomat.view.grouping.settings.GroupingColoring;

/**
 * Draws guiding lines for the mouse pointer.
 *
 * @version          080724
 * @author           Magnus Rosell
 */
public class Guide{

    private SparsePicture sparsePicture;
    private Graphics g;

    // The previous position
    private int oldX=-1;
    private int oldY=-1;

    // The rectangle start, or the start (x or y)-line
    private int rememberX=-1;
    private int rememberY=-1;

    private boolean draw=true; // If it should be drawed
    private boolean drawable=false; // If it's possible to draw it

    public Guide(SparsePicture sparsePicture){
	this.sparsePicture=sparsePicture;
	g=sparsePicture.getGraphics();
	//	enter();
    }

    public void toggleDraw(){
	if(draw){
	    reset();
	}
	setDraw(!draw);
    }

    private void setDraw(boolean draw){
	if(!draw && this.draw && oldX!=-1){
	    reset();
	}
	this.draw=draw;
    }
    
    protected void setRect(int rememberX, int rememberY){
	if(oldX!=-1 || oldY!=1){
	    unDraw();
	    oldX=-1;
	    oldY=-1;
	}
	this.rememberX=rememberX;
	this.rememberY=rememberY;
	draw(rememberX, rememberY);
    }

    protected void noRect(int x, int y){
	if(rememberX!=-1 && rememberY!=-1){
	    reset();
	    draw(x,y);
	}
    }

    protected void setRows(int rememberX, int rememberY){
	if(oldX!=-1 || oldY!=1){
	    unDraw();
	    oldX=-1;
	    oldY=-1;
	}
	this.rememberY=rememberY;
	draw(rememberX, rememberY);
    }

    protected void noRows(int x, int y){
	if(rememberX==-1 && rememberY!=-1){
	    reset();
	    draw(x,y);
	}
    }

    protected void setCols(int rememberX, int rememberY){
	if(oldX!=-1 || oldY!=1){
	    unDraw();
	    oldX=-1;
	    oldY=-1;
	}
	this.rememberX=rememberX;
	draw(rememberX, rememberY);
    }

    protected void noCols(int x, int y){
	if(rememberX!=-1 && rememberY==-1){
	    reset();
	    draw(x,y);
	}
    }




    protected void draw(int x, int y){
	if(!draw || !drawable)
	    return;
	
	g=sparsePicture.getGraphics();
	if(oldX!=-1){
	    unDraw();
	}
	g.setXORMode(GroupingColoring.getBackgroundColor());

	if(rememberX!=-1 && rememberY!=-1){
	    int firstX=rememberX;
	    int dx = x-rememberX;
	    if(x<rememberX){
		firstX=x;
		dx = rememberX-x;
	    }
	    int firstY=rememberY;
	    int dy = y-rememberY;
	    if(y<rememberY){
		firstY=y;
		dy = rememberY-y;
	    }
	    g.drawRect(firstX, firstY, dx, dy);
	}else if(rememberX!=-1){
	    g.drawLine(rememberX, 0, rememberX, sparsePicture.getHeight());
	    g.drawLine(x, 0, x, sparsePicture.getHeight());
	}else if(rememberY!=-1){
	    g.drawLine(0, rememberY, sparsePicture.getWidth(), rememberY);
	    g.drawLine(0, y, sparsePicture.getWidth(), y);
	}else{
	    g.drawLine(x, 0, x, sparsePicture.getHeight());
	    g.drawLine(0, y, sparsePicture.getWidth(), y);
	}

	oldX=x;
	oldY=y;
    }

    protected void unDraw(){
	if(!draw || !drawable)
	    return;

	g=sparsePicture.getGraphics();
	g.setXORMode(GroupingColoring.getBackgroundColor());

	if(rememberX!=-1 && rememberY!=-1){
	    int firstX=rememberX;
	    int dx = oldX-rememberX;
	    if(oldX<rememberX){
		firstX=oldX;
		dx = rememberX-oldX;
	    }
	    int firstY=rememberY;
	    int dy = oldY-rememberY;
	    if(oldY<rememberY){
		firstY=oldY;
		dy = rememberY-oldY;
	    }
	    g.drawRect(firstX, firstY, dx, dy);
	}else if(rememberX!=-1){
	    g.drawLine(rememberX, 0, rememberX, sparsePicture.getHeight());
	    g.drawLine(oldX, 0, oldX, sparsePicture.getHeight());
	}else if(rememberY!=-1){
	    g.drawLine(0, rememberY, sparsePicture.getWidth(), rememberY);
	    g.drawLine(0, oldY, sparsePicture.getWidth(), oldY);
	}else{
	    g.drawLine(oldX, 0, oldX, sparsePicture.getHeight());
	    g.drawLine(0, oldY, sparsePicture.getWidth(), oldY);
	}
    }

    protected void exit(){
	unDraw();
	oldX=-1;
	oldY=-1;
	drawable=false;
    }

    protected void enter(){
	drawable=true;
    }

    protected void reset(){
	unDraw();
	oldX=-1;
	oldY=-1;
	rememberX=-1;
	rememberY=-1;
    }

}
