package mro.util.experimentation;

import mro.util.*;
import mro.util.doublehashmap.*;

import java.io.*;
import java.util.*; 

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.SAXException;


/**
 * RelativeTable. A class for handling relative entries in a Result Table.
 *
 * @version          080505
 * @author           Magnus Rosell
 */

public class RelativeTable{

    private Vector<RelativeEntry> relativeVector = 
	new Vector<RelativeEntry>();
    
    public void add(RelativeEntry re){
	relativeVector.add(re);
    }

    public Vector<RelativeEntry> getRelativeVector(){
	return new Vector<RelativeEntry>(relativeVector);
    }

    public Vector<RelativeEntry> getRowRelativeVector(){
	Vector<RelativeEntry> ret = new Vector<RelativeEntry>();
	for(int i=0; i<relativeVector.size(); i++){
	    RelativeEntry re = relativeVector.elementAt(i);
	    if(re.getRelativeKind()==0)
		ret.add(re);
	}
	if(ret.size()>0)
	    return ret;
	return null;
    }

    public Vector<RelativeEntry> getColumnRelativeVector(){
	Vector<RelativeEntry> ret = new Vector<RelativeEntry>();
	for(int i=0; i<relativeVector.size(); i++){
	    RelativeEntry re = relativeVector.elementAt(i);
	    if(re.getRelativeKind()==1)
		ret.add(re);
	}
	if(ret.size()>0)
	    return ret;
	return null;
    }

    public Vector<RelativeEntry> getColumnRowRelativeVector(){
	Vector<RelativeEntry> ret = new Vector<RelativeEntry>();
	for(int i=0; i<relativeVector.size(); i++){
	    RelativeEntry re = relativeVector.elementAt(i);
	    if(re.getRelativeKind()==2)
		ret.add(re);
	}
	if(ret.size()>0)
	    return ret;
	return null;
    }

    public Vector<RelativeEntry> getRowColumnRelativeVector(){
	Vector<RelativeEntry> ret = new Vector<RelativeEntry>();
	for(int i=0; i<relativeVector.size(); i++){
	    RelativeEntry re = relativeVector.elementAt(i);
	    if(re.getRelativeKind()==3)
		ret.add(re);
	}
	if(ret.size()>0)
	    return ret;
	return null;
    }


    /********* IO *********/

    /**
     * Saves a Table-object to a file with the specified file name.
     */
    public void save(String fileName) throws IOException{
	save(new File(fileName));
    }

    /**
     * Saves a Table-object to the specified file.
     */
    public void save(File file) throws IOException{
	//	try {
	    File dir = file.getParentFile();
	    if(!dir.exists())
		dir.mkdirs();

	    // XSL
	    String xslFileName = file.getParent()+File.separator+
		"RelativeTable.xsl";

	    PrintWriter out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(xslFileName))));
	    printXSL(out);
	    out.close();

	    // XML
	    out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));

	    //            printXMLHead(out);
	    printXML(out);
	    out.close();
	    /*
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	    */
    }

    /**
     * Prints the complete DTD of a Table-XML-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printDTD(PrintWriter pw) throws IOException{
	pw.println("<!DOCTYPE relativetable [");
	printPartDTD(pw);

	pw.println("]>");
    }

    /**
     * Prints the DTD of a Table-XML-file to a 
     * {@link java.io.PrintWriter}. Only the elements.
     * This method may be used when the Table-XML should
     * be part of a larger XML.
     */
    public static void printPartDTD(PrintWriter pw) throws IOException{
	pw.println("<!--A relative table-->");
	pw.println("<!ELEMENT relativetable (rele+)>");

	pw.println("<!--A relative entry-->");
	pw.println("<!ELEMENT rele (dim, nr, relnr)>");

	pw.println("<!--The dimension, ie r(ow), c(olumn), cr (a column of row relative entries), rc (a row of column relative entries)-->");
	pw.println("<!ELEMENT dim (#PCDATA)>");

	pw.println("<!--The number to be compared-->");
	pw.println("<!ELEMENT nr (#PCDATA)>");

	pw.println("<!--The number to compare to-->");
	pw.println("<!ELEMENT relnr (#PCDATA)>");

    }

    /**
     * Prints a reference to the Table-XSL-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printTableXSLReference(PrintWriter pw) throws IOException{
	pw.println("<?xml-stylesheet type=\"text/xsl\" href=\"RelativeTable.xsl\"?>");
    }

    /**
     * Prints the Table to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXML(PrintWriter pw) throws IOException{
	XMLHelper.printXMLHead(pw);
	printTableXSLReference(pw); 
	printDTD(pw);
	printPartXML(pw);
    }

    public void printPartXML(PrintWriter pw) throws IOException{
	pw.println("<relativetable>");
	for(int i=0; i<relativeVector.size(); i++)
	    relativeVector.elementAt(i).printPartXML(pw);
	pw.println("</relativetable>");
   }


    /**
     * Prints a basic XSL-file for Table XML-files to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXSL(PrintWriter pw) throws IOException{
	pw.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
	pw.println("<xsl:stylesheet version=\"1.0\"");
	pw.println("xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">");
	pw.println("<xsl:template match=\"/\">");
	pw.println("<html>");
	pw.println("<body>");
	pw.println("<table border=\"1\">");

	pw.println("<xsl:apply-templates/>");

	pw.println("</table>");
	pw.println("</body>");
	pw.println("</html>");
	pw.println("</xsl:template>");


	pw.println("<xsl:template match=\"ptr\">");
	pw.println("<tr>");
	pw.println("<xsl:for-each select=\"ptc\">");
	pw.print("<td><xsl:value-of select=\".\"/></td>");
	pw.println("\n</xsl:for-each>");
	pw.println("</tr>");
	pw.println("</xsl:template>");

	pw.println("</xsl:stylesheet>");

    }

}

