package mro.util.experimentation;

import mro.util.measures.*;
import mro.util.properties.*;

/**
 * Class for storing info for each directory.
 *
 * @version          080506
 * @author           Magnus Rosell
 */
public class Result{
    protected String dirName;
    protected Measures measures;
    protected PropertiesPool pp;
    protected String measureAbbreviation;
    protected String roundString=null;

    public Result(String dirName, Measures measures, PropertiesPool pp, 
		  String measureAbbreviation, String roundString){
	this.dirName=dirName;
	this.measures=measures;
	this.pp=pp;
	this.measureAbbreviation=measureAbbreviation;
	this.roundString=roundString;
    }

    /**
     * Shallow copy constructor.
     */
    public Result(Result result, 
		  String measureAbbreviation, String roundString){
	this.dirName=result.dirName;
	this.measures=result.measures;
	this.pp=result.pp;
	this.roundString=roundString;
	this.measureAbbreviation=measureAbbreviation;
    }

    /**
     * Not possible if the Result is relative!
     * <br><br>
     * Does not keep the PropertiesPool as the Property:s of
     * the different Result:s may be different.
     * Does not change the measureAbbrevation.
     */
    public void add(Result res) throws MeasureNameConflictException{
	//	System.out.println("measures: "+measures+", "+measures.size());
	//	System.out.println("res.measures: "+res.measures+", "+res.measures.size());

	MeasureGroup mg = measures.constructAverageMeasures(res.measures);
	//	System.out.println("mg: "+mg);
	measures = new Measures(mg);
	pp=null; // The Property:s may be different.
    }

    /**
     * Returns the relative result of this compared to the specified Result.
     */ 
    public Result relative(Result relResult, String roundString){

	Measures relMeasures = null;
	try{
	    relMeasures = new Measures(measures.relative(relResult.measures));
	    //	    relMeasures = new Measures(measures.subtract(relResult.measures));
	    //	    relMeasures = new Measures(measures.divide(relResult.measures));
	}catch(MeasureNameConflictException mnce){
	    // Won't happen here
	}
	Result ret = new Result(null, relMeasures, null, 
				relResult.measureAbbreviation,
				roundString);
	return ret;
    }



}
