package mro.util.measures;

import java.io.*;
import java.util.*; 

/**
 * A group of Measure:s for the purpose of organization.
 *
 * @version          080507
 * @author           Magnus Rosell
 */
public class MeasureGroup extends Measure{

    /** All Measure:s. No MeasureGroup:s. */
    private HashMap<String, Measure> nameMeasureMap = 
	new HashMap<String, Measure>();

    /** MeasureGroup:s. Only top-level MeasureGroups. */
    private HashMap<String, MeasureGroup> nameMeasureGroupMap = 
	new HashMap<String, MeasureGroup>();

    /** 
     * The vector of Measures in the order the user wants it.
     * Only top-level Measure:s and MeasureGroup:s. 
     */
    private Vector<Measure> measureVector = new Vector<Measure>();

    // Used in Properties
    MeasureGroup(){}

    public MeasureGroup(String name){
	this.name=name;
    }

    /**
     * Copy Constructor. Deep. Makes copies of all Measure:s of all kinds.
     */
    public MeasureGroup(MeasureGroup mg) throws MeasureNameConflictException{
	this.name = mg.getName();
	Vector<Measure> vec = mg.getMeasureVector();
	for(int i=0; i<vec.size(); i++){
	    Measure m2 = vec.elementAt(i);
	    Measure newM = copy(m2);
	    addMeasure(newM);
	}
    }

    /**
     * Deep copy. Makes copies of all Measure:s of all kinds.
     */
    public static Measure copy(Measure m) throws MeasureNameConflictException{
	Measure ret = null;
	if(m instanceof MeasureGroup){
	    MeasureGroup mg =(MeasureGroup) m;
	    MeasureGroup newMG = new MeasureGroup(mg.getName());
	    Vector<Measure> vec = mg.getMeasureVector();
	    for(int i=0; i<vec.size(); i++){
		Measure m2 = vec.elementAt(i);
		Measure newM = copy(m2);
		newMG.addMeasure(newM);
	    }
	    ret = newMG;
	}else if(m instanceof AverageMeasureGroup){
	    ret = new AverageMeasureGroup((AverageMeasureGroup) m);
	}else if(m instanceof AverageMeasure){
	    ret = new AverageMeasure((AverageMeasure) m);
	}else{
	    ret = new Measure(m);
	}
	return ret;
    }


    public void addMeasureVector(Vector mv) 
	throws MeasureNameConflictException{
	for(int i=0; i<mv.size(); i++){
	    Measure m = (Measure) mv.elementAt(i);
	    addMeasure(m);
	}
    }

    /**
     * Adds a Measure of any kind.
     */
    public void addMeasure(Measure m) throws MeasureNameConflictException{
	if(m instanceof MeasureGroup){
	    MeasureGroup mg =(MeasureGroup) m;
	    if(getMeasureGroup(mg.getName())!=null){
		System.out.println(mg.getName());
		throw new MeasureNameConflictException("MeasureGroup: "+mg.getName());
	    }else{
		//		System.out.println("** "+getName()+", add: "+mg.getName());
		nameMeasureGroupMap.put(mg.getName(), mg);
	    }
	}else{
	    if(getMeasure(m.getName())!=null){
		throw new MeasureNameConflictException("Measure: "+m.getName());
	    }else{
		nameMeasureMap.put(m.getName(), m);
	    }
	}
	measureVector.add(m);
    }

    public MeasureGroup relative(MeasureGroup other){
	MeasureGroup ret = subtract(other);
	if(ret==null)
	    return null;
	ret = ret.divide(other);
	return ret;
    }

    public MeasureGroup subtract(MeasureGroup other){
	MeasureGroup ret = new MeasureGroup(getName());
	if(measureVector.size() != other.measureVector.size())
	    return ret;

	for(int i=0; i<measureVector.size(); i++){
	    Measure m = measureVector.elementAt(i);
	    Measure o = other.measureVector.elementAt(i);
	    //	    Measure o = other.getMeasure(m.getName());
	    if(o == null)
		return null;
	    Measure s = m.subtract(o);
	    if(m instanceof MeasureGroup)
		s = ((MeasureGroup) m).subtract((MeasureGroup) o);
	    else if(m instanceof AverageMeasureGroup)
		s = ((AverageMeasureGroup) m).subtract((AverageMeasureGroup) o);
	    else if(m instanceof AverageMeasure)
		s = ((AverageMeasure) m).subtract((AverageMeasure) o);
	    try{
		if(s!=null)
		    ret.addMeasure(s);
	    }catch(MeasureNameConflictException mnce){
		// Won't happen here
	    }
	}
	return ret;
    }

    public MeasureGroup divide(MeasureGroup other){
	MeasureGroup ret = new MeasureGroup(getName());
	if(measureVector.size() != other.measureVector.size())
	    return ret;

	for(int i=0; i<measureVector.size(); i++){
	    Measure m = measureVector.elementAt(i);
	    Measure o = other.measureVector.elementAt(i);
	    //	    Measure o = other.getMeasure(m.getName());
	    if(o == null)
		return null;
	    Measure s = m.divide(o);
	    if(m instanceof MeasureGroup)
		s = ((MeasureGroup) m).divide((MeasureGroup) o);
	    else if(m instanceof AverageMeasureGroup)
		s = ((AverageMeasureGroup) m).divide((AverageMeasureGroup) o);
	    else if(m instanceof AverageMeasure)
		s = ((AverageMeasure) m).divide((AverageMeasure) o);
	    try{
		if(s!=null)
		    ret.addMeasure(s);
	    }catch(MeasureNameConflictException mnce){
		// Won't happen here
	    }
	}
	return ret;
    }




    /**
     * Constructs a new MeasureGroup with AverageMeasure:s from the Measure:s
     * in this and the specified MeasureGroup. 
     * Deep. 
     */
    public MeasureGroup constructAverageMeasures(MeasureGroup mg) throws MeasureNameConflictException{
	return constructAverageMeasuresRecuresively(this, mg);
    }


    private HashMap<String, Measure> makeNameMeasureMap(Vector<Measure> vec){
	HashMap<String, Measure> nameMeasureMap = 
	    new HashMap<String, Measure>();
	for(int i=0; i<vec.size(); i++){
	    Measure m = vec.elementAt(i);
	    nameMeasureMap.put(m.getName(), m);
	}
	return nameMeasureMap;
    }

    //    private String indent = "";

    private MeasureGroup 
	constructAverageMeasuresRecuresively(MeasureGroup mg1, 
					     MeasureGroup mg2) throws MeasureNameConflictException{

	MeasureGroup ret = null;
	if(mg1!=null)
	    ret = new MeasureGroup(mg1.name);
	else if(mg2!=null)
	    ret = new MeasureGroup(mg2.name);
	
	HashMap<String, Measure> nameMeasureMap1 = null;
	if(mg1!=null){
	    Vector<Measure> vec1 = mg1.getMeasureVector();
	    nameMeasureMap1 = makeNameMeasureMap(vec1);
	}	
	HashMap<String, Measure> nameMeasureMap2 = null;
	if(mg2!=null){
	    Vector<Measure> vec2 = mg2.getMeasureVector();
	    nameMeasureMap2 = makeNameMeasureMap(vec2);
	}

	HashSet<String> allNames = new HashSet<String>();
	if(mg1!=null)
	    allNames.addAll(nameMeasureMap1.keySet());
	if(mg2!=null)
	    allNames.addAll(nameMeasureMap2.keySet());
	
	Iterator<String> it = allNames.iterator();
	while(it.hasNext()){
	    String name = it.next();
	    Measure m1 = null;
	    if(mg1!=null)
		m1 = nameMeasureMap1.get(name);
	    Measure m2 = null;
	    if(mg2!=null)
		m2 = nameMeasureMap2.get(name);

	    /*
	    if(m1!=null)
		System.out.println(indent+"m1: "+m1.getName());
	    if(m2!=null)
		System.out.println(indent+"m2: "+m2.getName());
	    */

	    if(m1==null || m2==null){
		Measure m = m1;
		if(m1==null)
		    m=m2;
		if(m instanceof MeasureGroup){
		    //		    String oldIndent = indent;
		    //		    indent = indent+" ";
		    MeasureGroup mg = constructAverageMeasuresRecuresively((MeasureGroup) m1, (MeasureGroup) m2);
		    //		    indent = oldIndent;
		    ret.addMeasure(mg);
		}else
		    ret.addMeasure(m);
	    }else if((m1 instanceof MeasureGroup) && 
		     (m2 instanceof MeasureGroup)){

		//		String oldIndent = indent;
		//		indent = indent+" ";
		MeasureGroup mg = constructAverageMeasuresRecuresively((MeasureGroup) m1, (MeasureGroup) m2);
		//		indent = oldIndent;
		if(mg!=null)
		    ret.addMeasure(mg);
	    }else if((!(m1 instanceof MeasureGroup) && (m2 instanceof MeasureGroup)) || (!(m2 instanceof MeasureGroup) && (m1 instanceof MeasureGroup))){
		return null;
	    }else if(m1 instanceof AverageMeasureGroup){
		if(m2 instanceof AverageMeasureGroup){
		    AverageMeasureGroup amg1 = (AverageMeasureGroup) m1;
		    AverageMeasureGroup amg2 = (AverageMeasureGroup) m2;
		    AverageMeasureGroup newAMG = 
			new AverageMeasureGroup(amg1);
		    newAMG.addAverageMeasureVector(amg2.getAverageMeasureVector());
		    ret.addMeasure(newAMG);
		}else if(m2 instanceof AverageMeasure){
		    AverageMeasureGroup amg1 = (AverageMeasureGroup) m1;
		    AverageMeasureGroup newAMG = 
			new AverageMeasureGroup(amg1);
		    newAMG.addAverageMeasure((AverageMeasure) m2);
		    ret.addMeasure(newAMG);
		}else
		    return null;
	    }else if(m2 instanceof AverageMeasureGroup){
		if(m2 instanceof AverageMeasure){
		    AverageMeasureGroup amg2 = (AverageMeasureGroup) m2;
		    AverageMeasureGroup newAMG = 
			new AverageMeasureGroup(amg2);
		    newAMG.addAverageMeasure((AverageMeasure) m2);
		    ret.addMeasure(newAMG);
		}else
		    return null;
	    }else if(m1 instanceof AverageMeasure){
		if(m2 instanceof AverageMeasure){
		    AverageMeasure am1 = 
			new AverageMeasure((AverageMeasure) m1);
		    AverageMeasure am2 = 
			new AverageMeasure((AverageMeasure) m2);
		    AverageMeasureGroup amg = 
			new AverageMeasureGroup(m1.getName(), m1.getDescription());
		    amg.addAverageMeasure(am1);
		    amg.addAverageMeasure(am2);
		    ret.addMeasure(amg);
		}else{
		    AverageMeasure am1 = 
			new AverageMeasure((AverageMeasure) m1);
		    am1.addMeasure(m2);
		    ret.addMeasure(am1);
		}
	    }else if(m2 instanceof AverageMeasure){
		AverageMeasure am2 = new AverageMeasure((AverageMeasure) m2);
		am2.addMeasure(m1);
		ret.addMeasure(am2);
	    }else{ // Measure
		AverageMeasure am = 
		    new AverageMeasure(m1.getName(), m1.getDescription());
		am.addMeasure(m1);
		am.addMeasure(m2);
		ret.addMeasure(am);
	    }
	}
	return ret;
    }


    /*
    private MeasureGroup 
	constructAverageMeasuresRecuresively(MeasureGroup mg1, 
					     MeasureGroup mg2) throws MeasureNameConflictException{

	MeasureGroup ret = new MeasureGroup(mg1.name);
	Vector<Measure> vec1 = mg1.getMeasureVector();
	Vector<Measure> vec2 = mg2.getMeasureVector();
	if(vec1.size() != vec2.size())
	    return ret;

	for(int i=0; i<vec1.size(); i++){
	    Measure m1 = vec1.elementAt(i);
	    Measure m2 = vec2.elementAt(i);

	    System.out.println(indent+"m1: "+m1.getName());
	    System.out.println(indent+"m2: "+m2.getName());

	    if(m1 instanceof MeasureGroup){
		if(!(m2 instanceof MeasureGroup))
		    return null;
		
		String oldIndent = indent;
		indent = indent+" ";
		MeasureGroup mg = constructAverageMeasuresRecuresively((MeasureGroup) m1, (MeasureGroup) m2);
		indent = oldIndent;
		if(mg!=null)
		    ret.addMeasure(mg);
	    }else if(m2 instanceof MeasureGroup){
		return null;
	    }else if(m1 instanceof AverageMeasure){
		AverageMeasure am1 = new AverageMeasure((AverageMeasure) m1);
		am1.addMeasure(m2);
		ret.addMeasure(am1);
	    }else if(m2 instanceof AverageMeasure){
		AverageMeasure am2 = new AverageMeasure((AverageMeasure) m2);
		am2.addMeasure(m1);
		ret.addMeasure(am2);
	    }else{ // Measure
		AverageMeasure am = 
		    new AverageMeasure(m1.getName(), m1.getDescription());
		am.addMeasure(m1);
		am.addMeasure(m2);
		ret.addMeasure(am);
	    }
	}
	return ret;
    }
*/

    /**
     * Returns a {@link MeasureGroup} with the specified name.
     * null if it doesn't exist.
     */
    public MeasureGroup getMeasureGroup(String measureGroupName){
	MeasureGroup pg = nameMeasureGroupMap.get(measureGroupName);
	return pg;
    }

    /**
     * Returns a {@link Measure} with the specified name.
     * null if it doesn't exist.
     */
    public Measure getMeasure(String name){
	Measure p = nameMeasureMap.get(name);
	return p;
    }

    /**
     * Sets a Measure with the given name to the given value.
     * If the Measure is not in the measures 
     * a {@link MeasureNotFoundException} is thrown.
     */
    /*
    public void setMeasure(String name, double value) 
	throws MeasureNotFoundException{
	Measure m = nameMeasureMap.get(name);
	if(m==null)
	    throw new MeasureNotFoundException();
	m.setValue(value);
    }
    */

    /*
    public double getMeasureValue(String name){
	return nameMeasureMap.get(name).getValue();
    }
    */

    public Vector<Measure> getMeasureVector(){
	return measureVector;
    }

    public int size(){
	return measureVector.size();
    }
	
    public void setName(String name){
	this.name=name;
    }

    public String getName(){
	return name;
    }

    /**
     * Sets all Measure "stamps" recursively. Use with caution.
     */
    public void setAllMeasuresStampsRecursively(String stamp){
	setAllMeasuresStampsRecursively(this, stamp);
    }

    private void setAllMeasuresStampsRecursively(Measure m, String stamp){
	if(m instanceof MeasureGroup){
	    Vector<Measure> vec = ((MeasureGroup) m).getMeasureVector();
	    for(int i=0; i<vec.size(); i++)
		setAllMeasuresStampsRecursively(vec.elementAt(i), stamp);
	}
	m.setStamp(stamp);
    }


    public void printXML(PrintWriter pw) throws IOException{
	pw.println("<mg>");

	pw.println("<n>"+getName()+"</n>");
	pw.println("<d>"+getDescription()+"</d>");
	pw.println("<stamp>"+getStamp()+"</stamp>");


	// Skriver alla underegenskaper
	for(int i=0; i<measureVector.size(); i++){
	    Measure m = (Measure) measureVector.elementAt(i);
	    m.printXML(pw);
	}
	pw.println("</mg>");
    }


    public String toString(){
	return name;
    }


}


